@@ -4,7 +4,6 @@ use tokio::net::TcpListener;
 use tokio_rustls::server::TlsStream;
 use tokio_rustls::TlsAcceptor;
 
-//use futures_util::future::TryFutureExt;
 use std::collections::HashMap;
 use std::future::Future;
 use std::io;
@@ -71,12 +70,10 @@ impl Server {
     pub async fn serve(
         self,
         cmap: HashMap<String, config::ServerCfg>,
-        default: String,
         handler: impl Handler + 'static + Copy,
     ) -> Result {
         for listen in self.listener {
             let cmap = cmap.clone();
-            let default = default.clone();
             let listen = Arc::new(listen);
             let acceptor = Arc::new(self.acceptor.clone());
 
@@ -86,8 +83,7 @@ impl Server {
                     let local_addr = stream.local_addr().unwrap();
                     let acceptor = acceptor.clone();
                     let cmap = cmap.clone();
-                    let default = default.clone();
-                    let mut handler = handler.clone();
+                    let mut handler = handler;
 
                     tokio::spawn(async move {
                         let mut stream = match acceptor.accept(stream).await {
@@ -105,7 +101,7 @@ impl Server {
 
                         let srv = match cmap.get(sni) {
                             Some(h) => h,
-                            None => cmap.get(&default).unwrap(),
+                            None => return Ok(()) as io::Result<()>,
                         }
                         .to_owned();
 
@@ -138,7 +134,7 @@ impl Server {
     }
 }
 
-pub async fn get_request(mut con: conn::Connection) -> Result<(conn::Connection, url::Url)> {
+async fn get_request(mut con: conn::Connection) -> Result<(conn::Connection, url::Url)> {
     let mut buffer = [0; 1024];
     let len = match tokio::time::timeout(
         tokio::time::Duration::from_secs(5),
@@ -169,9 +165,9 @@ pub async fn get_request(mut con: conn::Connection) -> Result<(conn::Connection,
         request = request.replacen("//", "gemini://", 1);
     }
 
-    if request.ends_with("\n") {
+    if request.ends_with('\n') {
         request.pop();
-        if request.ends_with("\r") {
+        if request.ends_with('\r') {
             request.pop();
         }
     }
@@ -187,36 +183,30 @@ pub async fn get_request(mut con: conn::Connection) -> Result<(conn::Connection,
         }
     };
 
-    match url.host_str() {
-        Some(h) => {
-            if con.srv.server.hostname.as_str() != h.to_lowercase() {
-                logger::logger(con.peer_addr, Status::ProxyRequestRefused, &url.as_str());
-                con.send_status(Status::ProxyRequestRefused, None)
-                    .await
-                    .map_err(|e| e.to_string())?;
-                return Err(Box::new(GemError("Wrong host".into())));
-            }
+    if let Some(h) = url.host_str() {
+        if con.srv.server.hostname.as_str() != h.to_lowercase() {
+            logger::logger(con.peer_addr, Status::ProxyRequestRefused, url.as_str());
+            con.send_status(Status::ProxyRequestRefused, None)
+                .await
+                .map_err(|e| e.to_string())?;
+            return Err(Box::new(GemError("Wrong host".into())));
         }
-        None => {}
     }
-    match url.port() {
-        Some(p) => {
-            if p != con.local_addr.port() {
-                logger::logger(con.peer_addr, Status::ProxyRequestRefused, &url.as_str());
-                con.send_status(Status::ProxyRequestRefused, None)
-                    .await
-                    .map_err(|e| e.to_string())?;
-            }
+    if let Some(p) = url.port() {
+        if p != con.local_addr.port() {
+            logger::logger(con.peer_addr, Status::ProxyRequestRefused, url.as_str());
+            con.send_status(Status::ProxyRequestRefused, None)
+                .await
+                .map_err(|e| e.to_string())?;
         }
-        None => {}
     }
     if url.scheme() != "gemini" {
-        logger::logger(con.peer_addr, Status::ProxyRequestRefused, &url.as_str());
+        logger::logger(con.peer_addr, Status::ProxyRequestRefused, url.as_str());
         con.send_status(Status::ProxyRequestRefused, None)
             .await
             .map_err(|e| e.to_string())?;
         return Err(Box::new(GemError("scheme not gemini".into())));
     }
 
-    return Ok((con, url));
+    Ok((con, url))
 }
